###################################################################
# 
# To be run with at least Python 3.7.1
# 
###################################################################

#%% [markdown]
#Packages:
import scipy.special as sc
from scipy.interpolate import interp1d
from scipy.signal import savgol_filter
import numpy as np
from numpy import loadtxt
import time
import timeit
import datetime
import os
import codecs # to open file not in UTF-8
import matplotlib as mpl
import matplotlib.pyplot as plt
import matplotlib.ticker as ticker
import re
from mpl_toolkits.axes_grid1 import AxesGrid
from IPython.display import Latex


#%% [markdown]
# ### To get the timestemp for saving
ts=time.time()
st=datetime.datetime.fromtimestamp(ts).strftime('%Y%m%d')
#%% [markdown]
# ### Definition of the working path and the division into directory and name
my_path = os.path.realpath(__file__) # Figures out the absolute path for you in case your working directory moves around.
my_dir = os.path.dirname(my_path)
my_name = os.path.splitext(os.path.basename(__file__))[0]

color_blue='#1f77b4'
color_lightblue='#aec7e8'
color_orange='#ff7f0e'
color_lightorange='#ffbb78'
color_blue1='#3182bd'
color_green2='#74c476'
color_orange3='#fdae6b'


plt.rcParams['image.cmap']='cividis'



#%%
# Defines how to load the data

def load_data(filename,data_y,switch_x_axis=False,skip_rows=1,data_x=None,):    
    data_enc = codecs.open(filename, encoding = 'cp1252')
    data = loadtxt(data_enc,skiprows=skip_rows,delimiter=',') # load data file ('filename'), how many rows at beginning are skipped ('skiprows', default = None), set delimiter (default: any whitespace)
    
    y = data[:, data_y]
    if data_x is not None:
        x = data[:, data_x]
                
        if switch_x_axis: # because some spectra were taken backwards
            return np.array([x[::-1],y[::-1]])
        else:
            return np.array([x,y])
    else:
        if switch_x_axis: # because some spectra were taken backwards
            return np.array(y[::-1])
        else:
            return np.array([y])


#%%

###################################################################
# 
# "Load Josephson data"
# 
###################################################################

DataForSimulationJosephson = load_data(os.path.join(my_dir,'180217PowerSweep40GHz200kOhmsOnPb(110)JosephsonBiasVoltageDetail00{:03}.export.csv'.format(0+1)),-1,True,skip_rows=2)


# load bias data (-> x axis)
biasJos = load_data(os.path.join(my_dir,'180217PowerSweep40GHz200kOhmsOnPb(110)JosephsonBiasVoltageDetail00001.export.csv'),0,True,skip_rows=2) # data


#%%
###################################################################
# 
# "Load MAR data"
# 
###################################################################


biasSimulationMAR = load_data(os.path.join(my_dir,'190905SplittingAndreevSweep01BiasSpectroscopyBiasSpectroscopy00{:03}.export.csv'.format(21)),0,True,skip_rows=2)

DataForSimulationMAR = load_data(os.path.join(my_dir,'190905SplittingAndreevSweep01BiasSpectroscopyBiasSpectroscopy00{:03}.export.csv'.format(21)),-2,True,skip_rows=2)



#%%
###################################################################
# 
# "Load Coherence data"
# 
###################################################################


# load bias data (-> x axis)
biasCoherence = load_data(os.path.join(my_dir,'180217PowerSweep40GHz200kOhmsOnPb(110)BiasSpectroscopy00001.export.csv'),0,True,2)
#%%
DataForSimulationCoherence = savgol_filter(load_data(os.path.join(my_dir,'180217PowerSweep40GHz200kOhmsOnPb(110)BiasSpectroscopy00001.export.csv'),-1,True,skip_rows=2), 15, 3)



#%%
# Definition of some values

Delta1 = 1.35 # in meV
Delta2 = 1.35 # in meV
DeltaTip = 1.35 # in meV
Epsilon0 = 0.248
R = 0.9
T = 0.000117885; # in eV (k_B * 1.368K = 0.117885meV*)
h = 4.135667662 *1.e-15
hbar = 6.582119514 *1.e-16
e = 1.602176634 *1.e-19
inverseConductanceQuantum = 12906.4037 # Ohms
NumberPhotons = 40.
Frequency = 40. * 1.e9 # in Hz
photonEnergy = h * Frequency # in eV
NormalStateDOS =0.025 #0.0000547722 #0.00002
RFAmplitude = 0

#%%
###################################################################
# 
# "Simulation Josephson"
# 
###################################################################
GQPZeroRFJos = interp1d(biasJos,DataForSimulationJosephson,kind='cubic',fill_value=4.7e-8,bounds_error=False)
FaktorPhotons = 1.5
def SpectrumJos(energy, RFAmplitude, NrElectrones):
    alphaPower = NrElectrones*RFAmplitude / photonEnergy
    NumberPhotons = np.ceil(alphaPower * FaktorPhotons)
    indPhotons = np.arange(-NumberPhotons,NumberPhotons+1,1)
    return np.dot(GQPZeroRFJos(np.add.outer(energy , indPhotons * photonEnergy/NrElectrones)) , (sc.jv(indPhotons, alphaPower))**2)

fig = plt.figure()
gs = mpl.gridspec.GridSpec(1,1)#,width_ratios=[1.5, 2, 2])
ax=plt.subplot(gs[0,0])#, sharey=axJosSim)
x1 = np.linspace(start=-0.007, stop=0.007, num=1001)
ax.plot(x1*10**3, SpectrumJos(x1,0.002,1), lw=2, color=color_blue1, label=r"$0$", zorder=5)
plt.show()

fig = plt.figure()
gs = mpl.gridspec.GridSpec(1,1)#,width_ratios=[1.5, 2, 2])
ax=plt.subplot(gs[0,0])#, sharey=axJosSim)
x1 = np.linspace(start=-0.007, stop=0.007, num=1001)
ax.plot(x1*10**3, SpectrumJos(x1,0.002,2), lw=2, color=color_blue1, label=r"$0$", zorder=5)
plt.show()


###################################################################
# 
# "Simulation MAR"
# 
###################################################################
GQPZeroRFMAR = interp1d(biasSimulationMAR,DataForSimulationMAR,kind='cubic',fill_value=4.7e-8,bounds_error=False)
FaktorPhotons = 1.5
def SpectrumMAR(energy, RFAmplitude,NrElectrones):
    alphaPower = NrElectrones*RFAmplitude / photonEnergy
    NumberPhotons = np.ceil(alphaPower * FaktorPhotons)
    indPhotons = np.arange(-NumberPhotons,NumberPhotons+1,1)
    return np.dot(GQPZeroRFMAR(np.add.outer(energy , indPhotons * photonEnergy/NrElectrones)) , (sc.jv(indPhotons, alphaPower))**2)

fig = plt.figure()
gs = mpl.gridspec.GridSpec(1,1)#,width_ratios=[1.5, 2, 2])
ax=plt.subplot(gs[0,0])#, sharey=axMARSim)
x1 = np.linspace(start=-0.007, stop=0.007, num=1001)
ax.plot(x1*10**3, SpectrumMAR(x1,0.002,1), lw=2, color=color_blue1, label=r"$0$", zorder=5)
plt.show()

fig = plt.figure()
gs = mpl.gridspec.GridSpec(1,1)#,width_ratios=[1.5, 2, 2])
ax=plt.subplot(gs[0,0])#, sharey=axMARSim)
x1 = np.linspace(start=-0.007, stop=0.007, num=1001)
ax.plot(x1*10**3, SpectrumMAR(x1,0.002,2), lw=2, color=color_blue1, label=r"$0$", zorder=5)
plt.show()

#%%
###################################################################
# 
# "Simulation Coherence"
# 
###################################################################
GQPZeroRFCoherence = interp1d(biasCoherence,DataForSimulationCoherence,kind='cubic',fill_value=4.7e-15,bounds_error=False)
FaktorPhotons = 1.5
def SpectrumCoherence(energy, RFAmplitude):
    alphaPower = RFAmplitude / photonEnergy
    NumberPhotons = np.ceil(alphaPower * FaktorPhotons)
    indPhotons = np.arange(-NumberPhotons,NumberPhotons+1,1)
    return np.dot(GQPZeroRFCoherence(np.add.outer(energy , indPhotons * photonEnergy)) , (sc.jv(indPhotons, alphaPower))**2)

fig = plt.figure()
gs = mpl.gridspec.GridSpec(1,1)#,width_ratios=[1.5, 2, 2])
ax=plt.subplot(gs[0,0])#, sharey=axCohSim)
x1 = np.linspace(start=0.0025, stop=0.0045, num=401)
ax.plot(x1*10**3, SpectrumCoherence(x1,0.00001), lw=2, color=color_blue1, label=r"$0$", zorder=5)

fig = plt.figure()
gs = mpl.gridspec.GridSpec(1,1)#,width_ratios=[1.5, 2, 2])
ax=plt.subplot(gs[0,0])#, sharey=axCohSim)
x1 = np.linspace(start=-0.0045, stop=-0.0025, num=401)
ax.plot(biasCoherence, DataForSimulationCoherence, lw=2, color=color_blue1, label=r"$0$", zorder=5,)

ax.set_xlim(+0.0025, +0.0045)
plt.show()


#%% [markdown]
# conversion to real physical units
#%%
def RSVoltageToPower(voltage):
    return 20*np.log10(voltage)
def PowerToVolt(x,R=50):
    return np.sqrt(10**(x/10)*2*R*0.001)

def HFPowerToRealAmplitude(x,damping,prefactor=10**3):
    return PowerToVolt(RSVoltageToPower(x)-damping)*prefactor

def VoltageAtJunction(NrSpectrum,PowerFirstSpectrum,PowerIncrease,Damping,prefactor=10**3):
    return PowerToVolt(RSVoltageToPower(PowerFirstSpectrum + (NrSpectrum - 1) * PowerIncrease) - Damping) * prefactor



#%%
###################################################################
# 
# "General definitions of the Figure"
# 
###################################################################

def cm2inch(value):
    return value/2.54

def GoldenerSchnitt(value):
    return value/1.6180339887


FigWidth = 17.8
FigHeight = GoldenerSchnitt(1.75*FigWidth)
#SavingPrefix = 'FigS_ExperimentVsSimulation'

fig = plt.figure(figsize=(cm2inch(FigWidth/2),cm2inch(FigHeight)),constrained_layout=False)
gs = mpl.gridspec.GridSpec(4,1,wspace=0.06,hspace=0.18,left=0.06, right=1.02,top=0.99,bottom=0.045,figure=fig)



###################################################################
# 
# "Josephson Simulation with CP"
# 
###################################################################
nJos = 100 # number of files 
axJosSim = fig.add_subplot(gs[2, 0])


Damping = 57.25


xscale = 10**3
yscale = 10**3
zscale = 10**6

x_min=-0.001
x_max=0.001
y_min=VoltageAtJunction(1,0.1,0.025,Damping)/yscale
y_max=VoltageAtJunction(nJos,0.1,0.025,Damping)/yscale


x1 = np.linspace(start=x_min, stop=x_max, num=201)
y = np.linspace(start=y_min, stop=y_max, num=151)
z = np.array([zscale*SpectrumJos(x1,j,2)  for j in y])#10/(k**2+10)
Z = z.reshape(151,201)
im = axJosSim.imshow(Z, interpolation='none',origin='lower',extent=[x_min*xscale,x_max*xscale,y_min*yscale,y_max*yscale],aspect='auto',vmin=-0.20, vmax=0.600)


axJosSim.set_ylim(bottom=0,top=y_max*yscale)  # outliers only

#axJosSim.set_xlabel("Sample bias (mV)") # x axis label 
#axJosSim.set_ylabel("HF amplitude (mV)") # y axis label


cb = plt.colorbar(im, fraction = 0.15, pad = 0.02, aspect=5.5, orientation='vertical')
cb.set_ticks([])
cb.ax.text(0.5, 0.5,r' $\mathrm{d}I/\mathrm{d}V$  ($\mu$S)', verticalalignment='center', horizontalalignment='center',color='white',rotation=-90, transform=cb.ax.transAxes)
cb.ax.text(0.5, 0, r' $-0.1$', verticalalignment='bottom', horizontalalignment='center',color='white', transform=cb.ax.transAxes)
cb.ax.text(0.5, 1, r' $0.3$', verticalalignment='top', horizontalalignment='center',color='black', transform=cb.ax.transAxes)
plt.minorticks_on()

axJosSim.xaxis.set_minor_locator(mpl.ticker.AutoMinorLocator(n=5))
axJosSim.yaxis.set_minor_locator(mpl.ticker.AutoMinorLocator(n=2))
start, end = axJosSim.get_xlim()
axJosSim.xaxis.set_ticks(np.arange(start, end+0.00001, 0.5))



###################################################################
# 
# "MAR simulation"
# 
###################################################################
nMAR = 61 # number of files 

axMARSim = fig.add_subplot(gs[3, 0])

Damping = 53.25

xscale = 10**3
yscale = 10**3
zscale = 10**0

x_min=-0.0022
x_max=0.0022
y_min=VoltageAtJunction(1,0.1,0.02,Damping)/yscale
y_max=VoltageAtJunction(nMAR,0.1,0.02,Damping)/yscale

x1 = np.linspace(start=x_min, stop=x_max, num=201)
y = np.linspace(start=y_min, stop=y_max, num=151)
z = np.array([zscale*SpectrumMAR(x1,j,2)  for j in y])#10/(k**2+10)
Z = z.reshape(151,201)
im = axMARSim.imshow(Z, interpolation='none',origin='lower',extent=[x_min*xscale,x_max*xscale,y_min*yscale,y_max*yscale],aspect='auto',vmin=-0.20, vmax=0.200)


axMARSim.set_ylim(0,y_max*yscale)  # outliers only

axMARSim.set_xlabel("Sample bias (mV)") # x axis label 
#axMARSim.set_ylabel("HF amplitude (mV)") # y axis label

cb = plt.colorbar(im, fraction = 0.15, pad = 0.02, aspect=5.5, orientation='vertical')
cb.set_ticks([])
cb.ax.text(0.5, 0.5,r' $\mathrm{d}^2 I/\mathrm{d}V^2$  (arb. u.)', verticalalignment='center', horizontalalignment='center',color='white',rotation=-90, transform=cb.ax.transAxes)
plt.minorticks_on()

axMARSim.yaxis.set_minor_locator(mpl.ticker.AutoMinorLocator(n=2))



###################################################################
# 
# "Coherence simulation"
# 
###################################################################
nCoherenceZoom=125


axCohSim = fig.add_subplot(gs[1, 0])

Damping = 56.6

xscale = 10**3
yscale = 10**3
zscale = 10**6


start, end = [-4.3,-1.3]
x_min=start/xscale
x_max=end/xscale
y_min=VoltageAtJunction(1,0.1,0.02,Damping)/yscale
y_max=VoltageAtJunction(nCoherenceZoom,0.1,0.025,Damping)/yscale

x1 = np.linspace(start=x_min, stop=x_max, num=201)
y = np.linspace(start=y_min, stop=y_max, num=151)
z = np.array([zscale*SpectrumCoherence(x1,j)  for j in y])#10/(k**2+10)
Z = z.reshape(151,201)
im = axCohSim.imshow(Z, interpolation='none',origin='lower',extent=[x_min*xscale,x_max*xscale,y_min*yscale,y_max*yscale],aspect='auto',vmin=0, vmax=0.300)

start, end = [-4.3,-1.3]
axCohSim.set_xlim(start,end)  # outliers only

axCohSim.set_ylim(0,y_max*yscale)  # outliers only

#axCohSim.set_xlabel("Sample bias (mV)") # x axis label 
#axCohSim.set_ylabel("HF amplitude (mV)") # y axis label

cb = plt.colorbar(im, fraction = 0.15, pad = 0.02, aspect=5.5, orientation='vertical')
cb.set_ticks([])
cb.ax.text(0.5, 0.5,r' $\mathrm{d}I/\mathrm{d}V$  (nS)', verticalalignment='center', horizontalalignment='center',color='white',rotation=-90, transform=cb.ax.transAxes)
cb.ax.text(0.5, 0, r' $0$', verticalalignment='bottom', horizontalalignment='center',color='white', transform=cb.ax.transAxes)
cb.ax.text(0.5, 1, r' $0.15$', verticalalignment='top', horizontalalignment='center',color='black', transform=cb.ax.transAxes)
plt.minorticks_on()

axCohSim.yaxis.set_minor_locator(mpl.ticker.AutoMinorLocator(n=2))
start, end = axCohSim.get_ylim()
axCohSim.yaxis.set_ticks(np.arange(start, end+0.00001, 0.2))


###################################################################
# 
# "Coherence simulation full"
# 
###################################################################
nCoherence = 330 # number of files 
axCohFullSim = fig.add_subplot(gs[0, 0])

Damping = 56.85161037520744


xscale = 10**3
yscale = 10**3
zscale = 10**6

x_min=-0.007
x_max=0.007
y_min=VoltageAtJunction(1,0.1,0.025,Damping)/yscale
y_max=VoltageAtJunction(nCoherence,0.1,0.025,Damping)/yscale

x1 = np.linspace(start=x_min, stop=x_max, num=201)
y = np.linspace(start=y_min, stop=y_max, num=151)
z = np.array([zscale*SpectrumCoherence(x1,j)  for j in y])#10/(k**2+10)
Z = z.reshape(151,201)
im = axCohFullSim.imshow(Z, interpolation='none',origin='lower',extent=[x_min*xscale,x_max*xscale,y_min*yscale,y_max*yscale],aspect='auto',vmin=-0.020, vmax=0.300)

axCohFullSim.set_ylim(0,y_max*yscale)  # outliers only

#axCohFullSim.set_xlabel("Sample bias (mV)") # x axis label 
#axCohFullSim.set_ylabel("HF amplitude (mV)") # y axis label

cb = plt.colorbar(im, fraction = 0.15, pad = 0.02, aspect=5.5, orientation='vertical')
cb.set_ticks([])
cb.ax.text(0.5, 0.5,r' $\mathrm{d}I/\mathrm{d}V$  (nS)', verticalalignment='center', horizontalalignment='center',color='white',rotation=-90, transform=cb.ax.transAxes)
cb.ax.text(0.5, 0, r' $-0.01$', verticalalignment='bottom', horizontalalignment='center',color='white', transform=cb.ax.transAxes)
cb.ax.text(0.5, 1, r' $0.15$', verticalalignment='top', horizontalalignment='center',color='black', transform=cb.ax.transAxes)
plt.minorticks_on()

axCohFullSim.yaxis.set_minor_locator(mpl.ticker.AutoMinorLocator(n=2))
start, end = axCohFullSim.get_ylim()
axCohFullSim.yaxis.set_ticks(np.arange(start, end+0.00001, 0.5))



###################################################################
# 
# "abcdefg"
# 
###################################################################
color_label='white'
position_x_label=0.025
position_y_label=0.95
# axJosMeas.text(position_x_label,position_y_label,r'{ (e)}',bbox=None,color=color_label, horizontalalignment='left',verticalalignment='top', weight='bold' , transform=axJosMeas.transAxes)
axJosSim.text(position_x_label,position_y_label,r'(f)',bbox=None,color=color_label, horizontalalignment='left',verticalalignment='top', weight='bold' , transform=axJosSim.transAxes)
# axMARMeas.text(position_x_label,position_y_label,r'{ (g)}',bbox=None,color=color_label, horizontalalignment='left',verticalalignment='top', weight='bold' , transform=axMARMeas.transAxes)
axMARSim.text(position_x_label,position_y_label,r'(h)',bbox=None,color=color_label, horizontalalignment='left',verticalalignment='top', weight='bold' , transform=axMARSim.transAxes)
# axCohMeas.text(position_x_label,position_y_label,r'{ (c)}',bbox=None,color=color_label, horizontalalignment='left',verticalalignment='top', weight='bold' , transform=axCohMeas.transAxes)
axCohSim.text(position_x_label,position_y_label,r'(d)',bbox=None,color=color_label, horizontalalignment='left',verticalalignment='top', weight='bold' , transform=axCohSim.transAxes)
# axCohFullMeas.text(position_x_label,position_y_label,r'{ (a)}',bbox=None,color=color_label, horizontalalignment='left',verticalalignment='top', weight='bold' , transform=axCohFullMeas.transAxes)
axCohFullSim.text(position_x_label,position_y_label,r'(b)',bbox=None,color=color_label, horizontalalignment='left',verticalalignment='top', weight='bold' , transform=axCohFullSim.transAxes)


###################################################################
# 
# "Ending and Save"
# 
###################################################################


# plt.savefig(os.path.join(my_dir,SavingPrefix+'.pdf'),dpi=300)#, bbox_inches = "tight")
# plt.savefig(os.path.join(my_dir,SavingPrefix+'.png'),dpi=300)#, bbox_inches = "tight",transparent=True)
plt.show()




# %%
