import sys
import math
import pickle
import numpy as np
import time
import scipy
import scipy.integrate
import scipy.special
import os
import matplotlib.pyplot as plt

######## Parameters for the simulation in Fig. 3 ######## 

V1 = np.arange(  0  ,  1.5 , 1.5/150) # Radiation energy in meV
V  = np.arange(-2.5,2.5,5/400) # Bias voltage in meV 

nu0t = 0.0001*8.1     # $\nu_0t$ value for the tip being FAR
nu0t = 0.0001*8.1*39  # $\nu_0t$ value for the tip being CLOSE

T    = 0.12         # Temperature in meV
Ga   = 0.02         # Dynes parameter in meV

freq = 40           # HF frequency in GHz 
hbarOmega = 2*np.pi*freq*6.582119514/10000  # photon energy in meV

u2 = 0.21        # $u^2/\nu_0t in meV$ 
v2 = 0.83        # $v^2/\nu_0t in meV$

Dtip = 1.35      # Superconducting gap in meV
e0 = 0.248       # YSR state energy in meV

G1 = 0.000700    # $\Gamma_1$ in meV
G2 = 0.000105    # $\Gamma_2$ in meV 


######## Parameters for Fig. S9 in the Supplement ######## 

V1 = np.arange(  0  ,  1.5 , 1.5/150) # Radiation energy in meV
V  = np.arange(-2.5,0,2.5/400) # Bias voltage in meV 

nu0t = 0.0001*8.1*45  # $\nu_0t$ value for the tip being CLOSE

T    = 0.12         # Temperature in meV
Ga   = 0.01         # Dynes parameter in meV

freq = 40           # HF frequency in GHz 
hbarOmega = 2*np.pi*freq*6.582119514/10000  # photon energy in meV

u2 = 0.59        # $u^2/\nu_0t in meV$ 
v2 = 2.25        # $v^2/\nu_0t in meV$

Dtip = 1.35      # Superconducting gap in meV

e0 = 1.50 * hbarOmega     # YSR state energy in meV
e0 = 1.25 * hbarOmega     # YSR state energy in meV

G1 = 0.000658    # $\Gamma_1$ in meV
G2 = 0.000082    # $\Gamma_2$ in meV 


################ First definitions: calculation of currents ################

# Tip GreensFucntion: the imaginary part of it corresponds to the density of states (Im(SC_GF) ).
def SC_GF(w): # this way of writing is essentially equivalent to Eq. (S35) for small Ga
    aa = w+1j*Ga
    return w/np.sqrt( Dtip**2 - aa * aa )

# Fermi distribution
def nf(w):
    return 1/(1+np.exp(w/T))

# Gamma_e: electron tunneling rate in the presence of radiation --- Eq. (S31)
def Ge(w,V,V1,nu0t, SC_GF_vec=None): 
    x = V1/hbarOmega
    lim = max([int(3*x),10]) # maximum number of photons
    
    ks = np.arange(-lim,lim+1,1)
    if SC_GF_vec==None:
        SC_GF_vec=[SC_GF(w-V-ks[k]*hbarOmega) for k in range(0,len(ks))]
    
    summ = 2*np.pi*u2*nu0t**2* np.dot( np.power(scipy.special.jv(ks,x),2),np.imag(SC_GF_vec))
    
    return summ, SC_GF_vec

# Gamma_h: hole tunneling rate in the presence of radiation  --- Eq. (S31) for holes
def Gh(w,V,V1,nu0t,SC_GF_vec=None):
    x = V1/hbarOmega
    lim = max([int(3*x),10])
 
    ks=np.arange(-lim,lim+1,1)
    if SC_GF_vec == None:
        SC_GF_vec=[SC_GF(w+V+ks[k]*hbarOmega) for k in range(0,len(ks))]
   
    summ = 2*np.pi*v2*nu0t**2* np.dot( np.power(scipy.special.jv(ks,x),2) , np.imag(SC_GF_vec))

    return summ, SC_GF_vec


# Gamma_e^{n_F}:  electron tunneling rate in the presence of radiation including occupation   --- Eq. (S32)
def Genf(w,V,V1,nu0t, SC_GF_vec=None, nf_vece=None):
    x = V1/hbarOmega
    lim = max([int(3*x),10])
    
    ks = np.arange(-lim,lim+1,1)   
    if SC_GF_vec == None:
        SC_GF_vec = [SC_GF(w-V-ks[k]*hbarOmega) for k in range(0,len(ks))]
    if nf_vece == None:
        nf_vece   = [   nf(w-V-ks[k]*hbarOmega) for k in range(0,len(ks))]
    
    summ = 2*np.pi*u2*nu0t**2*np.dot( np.power(scipy.special.jv(ks,x),2) , np.imag(SC_GF_vec)*nf_vece )
    
    return summ, SC_GF_vec, nf_vece

# Gamma_h^{n_F}:  hole tunneling rate in the presence of radiation including occupation   --- Eq. (S32) for holes 
def Ghnf(w,V,V1,nu0t, SC_GF_vec=None, nf_vech=None):
    x = V1/hbarOmega
    lim = max([int(3*x),10])
 
    ks=np.arange(-lim,lim+1,1)
    if SC_GF_vec==None:
        SC_GF_vec=[SC_GF(w+V+ks[k]*hbarOmega) for k in range(0,len(ks))]
    if nf_vech == None:
        nf_vech = [   nf(w+V+ks[k]*hbarOmega) for k in range(0,len(ks))]
    
    summ = 2*np.pi*v2*nu0t**2*np.dot( np.power(scipy.special.jv(ks,x),2) , np.imag(SC_GF_vec)*nf_vech )

    return summ, SC_GF_vec, nf_vech


# These two functions calculate the current
#def integrandS(w,V,V1,nu0t):
#
#    Ge_aux, SC_GF_vece   = Ge(  w,V,V1,nu0t )
#    Gh_aux, SC_GF_vech   = Gh(  w,V,V1,nu0t )
#    Genf_aux, SC_GF_vece = Genf(w,V,V1,nu0t, SC_GF_vece )
#    Ghnf_aux, SC_GF_vech = Ghnf(w,V,V1,nu0t, SC_GF_vech )
#   
#    return (1/((w-e0)**2 + (G1 + G2 + Ge_aux + Gh_aux)**2/4))*(G1*(Genf_aux - Ghnf_aux) - G2*(Ge_aux - Gh_aux - Genf_aux + Ghnf_aux))
#
#def integrandA(w,V,V1,nu0t):
#    Ge_aux, SC_GF_vece   = Ge(  w,V,V1,nu0t )
#    Gh_aux, SC_GF_vech   = Gh(  w,V,V1,nu0t )
#    Genf_aux, SC_GF_vece = Genf(w,V,V1,nu0t, SC_GF_vece )
#    Ghnf_aux, SC_GF_vech = Ghnf(w,V,V1,nu0t, SC_GF_vech )
#
#    return (2/((w-e0)**2 + (G1 + G2 + Ge_aux + Gh_aux)**2/4))*(Gh_aux*Genf_aux - Ge_aux*Ghnf_aux)


################ End first definitions: calculation of currents ################


################ Second definitions: calculation of dI/dV ################


# Here the derivatives of the previous functions start. Same order
def dSC_GF(w):   # Derivative of Tip GreensFucntion:
    aa = w + 1j*Ga 
    denom = np.sqrt(Dtip**2 - aa*aa)
    return (1+w*w/(denom*denom))/denom

def dnf(w):
    return -(1/T)*np.exp(w/T)/np.power(1+np.exp(w/T),2)

def dGe(w,V,V1,nu0t,dSC_GF_vec=None):
    x = V1/hbarOmega
    lim = max([int(3*x),10])

    ks = np.arange(-lim,lim+1,1)
    if dSC_GF_vec==None:
        dSC_GF_vec=[dSC_GF(w-V-ks[k]*hbarOmega) for k in range(0,len(ks))]
 
    summ =-2*np.pi*u2*nu0t**2* np.dot( np.power(scipy.special.jv(ks,x),2) , np.imag(dSC_GF_vec) )

    return summ, dSC_GF_vec

def dGh(w,V,V1,nu0t,dSC_GF_vec=None):
    x = V1/hbarOmega
    lim = max([int(3*x),10])
    
    ks = np.arange(-lim,lim+1,1)
    if dSC_GF_vec==None:
        dSC_GF_vec=[dSC_GF(w+V+ks[k]*hbarOmega) for k in range(0,len(ks))]
  
    summ = +2*np.pi*v2*nu0t**2* np.dot( np.power(scipy.special.jv(ks,x),2) , np.imag(dSC_GF_vec))

    return summ, dSC_GF_vec

def dGenf(w,V,V1,nu0t,SC_GF_vec=None,dSC_GF_vec=None,nf_vece=None,dnf_vece=None):
    x = V1/hbarOmega
    lim = max([int(3*x),10])
    
    ks = np.arange(-lim,lim+1,1)   
    if SC_GF_vec == None:
        SC_GF_vec  = [  SC_GF(w-V-ks[k]*hbarOmega) for k in range(0,len(ks))]
    if dSC_GF_vec == None:
        dSC_GF_vec = [ dSC_GF(w-V-ks[k]*hbarOmega) for k in range(0,len(ks))]
    if nf_vece == None:
        nf_vece    = [     nf(w-V-ks[k]*hbarOmega) for k in range(0,len(ks))]
    if dnf_vece == None:
        dnf_vece   = [    dnf(w-V-ks[k]*hbarOmega) for k in range(0,len(ks))]

    summ = -2*np.pi*u2*nu0t**2* np.dot( np.power(scipy.special.jv(ks,x),2) , np.imag(dSC_GF_vec)*nf_vece+ np.imag(SC_GF_vec)*dnf_vece)

    return summ, SC_GF_vec, dSC_GF_vec

def dGhnf(w,V,V1,nu0t,SC_GF_vec=None,dSC_GF_vec=None,nf_vech=None,dnf_vech=None):
    x = V1/hbarOmega
    lim = max([int(3*x),10])
    
    ks = np.arange(-lim,lim+1,1)
    if SC_GF_vec  == None:
       SC_GF_vec  = [  SC_GF(w+V+ks[k]*hbarOmega) for k in range(0,len(ks))]
    if dSC_GF_vec == None:
       dSC_GF_vec = [ dSC_GF(w+V+ks[k]*hbarOmega) for k in range(0,len(ks))]
    if nf_vech == None:
        nf_vech   = [     nf(w+V+ks[k]*hbarOmega) for k in range(0,len(ks))]
    if dnf_vech == None:
        dnf_vech  = [    dnf(w+V+ks[k]*hbarOmega) for k in range(0,len(ks))]

    summ = +2*np.pi*v2*nu0t**2* np.dot( np.power(scipy.special.jv(ks,x),2) , np.imag(dSC_GF_vec)*nf_vech+np.imag(SC_GF_vec)*dnf_vech)

    return summ, SC_GF_vec, dSC_GF_vec

# This  function calculates the inetrands of the conductance
def dallintegrands(w,V,V1,nu0t):
    Ge_aux,    SC_GF_vece  = Ge(   w,V,V1,nu0t)
    Gh_aux,    SC_GF_vech  = Gh(   w,V,V1,nu0t)
    Genf_aux,  SC_GF_vece, nf_vece  = Genf( w,V,V1,nu0t,SC_GF_vece)
    Ghnf_aux,  SC_GF_vech, nf_vech  = Ghnf( w,V,V1,nu0t,SC_GF_vech)
    dGe_aux,  dSC_GF_vece  = dGe(  w,V,V1,nu0t)
    dGh_aux,  dSC_GF_vech  = dGh(  w,V,V1,nu0t)
    dGenf_aux, SC_GF_vece, dSC_GF_vece = dGenf(w,V,V1,nu0t,SC_GF_vece,dSC_GF_vece,nf_vece)
    dGhnf_aux, SC_GF_vech, dSC_GF_vech = dGhnf(w,V,V1,nu0t,SC_GF_vech,dSC_GF_vech,nf_vech)

    G = G1 + G2 + Ge_aux + Gh_aux
    denom  = (1/((w-e0)**2 + G**2/4))
    ddenom = -((0.5*(dGe_aux+dGh_aux)*G)/np.power((w-e0)**2 + G**2/4,2))

    dIsG1 =  G1*( ddenom * (Genf_aux - Ghnf_aux)                   + denom*(dGenf_aux - dGhnf_aux))
    dIsG2 = -G2*( ddenom * (Ge_aux - Gh_aux - Genf_aux + Ghnf_aux) + denom*(dGe_aux - dGh_aux - dGenf_aux + dGhnf_aux))
    dIa   =     2*ddenom * (Gh_aux*Genf_aux - Ge_aux*Ghnf_aux)    + 2*denom*(dGh_aux*Genf_aux + Gh_aux*dGenf_aux - dGe_aux*Ghnf_aux - Ge_aux*dGhnf_aux)

    return np.array( [dIsG1 , dIsG2 , dIa] )

################ End second definitions: calculation of dI/dV ################


start_time = time.time()

############### dI/dV ###################


dIsG1 = np.zeros((len(V1),len(V))) # Arrays to store output
dIsG2 = np.zeros((len(V1),len(V)))
dIa   = np.zeros((len(V1),len(V)))


dw = 0.0002 # make sure that the integratios step is smaller than the smallest width
w = np.arange(e0-0.2,e0+0.2,dw) # integration range

for j in range(0,len(V1)):
    for i in range(0,len(V)):
        aux = dallintegrands(w,V[i],V1[j],nu0t)

        dIsG1[j,i] = np.sum(aux[0]) # This sums are rudimentary integrals
        dIsG2[j,i] = np.sum(aux[1])
        dIa[j,i]   = np.sum(aux[2])
        

dIsG1 = dIsG1 * dw # Multiply by the integration step
dIsG2 = dIsG2 * dw
dIa   = dIa   * dw


####### SAVING DATA FOR THE FIGS
with open("data.pickle","wb") as f:
    pickle.dump([dIsG1,dIsG2,dIa,V,V1,Ga,T,freq,nu0t,G1,G2,e0,Dtip,u2,v2],f,protocol=2)
#######


end_time = time.time()
print("--- %s seconds" % (end_time-start_time))




